from . import lib

import pyblish.lib
import pyblish.compat
from nose.tools import (
    with_setup
)


@with_setup(lib.setup, lib.teardown)
def test_compat():
    """Using compatibility functions works"""
    pyblish.compat.sort([])
    pyblish.compat.deregister_all()



"""Conveinence functions for general publishing"""

from __future__ import absolute_import

# Standard library
import logging
import warnings

# Local library
from . import api, logic, plugin, lib

log = logging.getLogger("pyblish.util")


def publish(context=None, plugins=None, targets=[]):
    """Publish everything

    This function will process all available plugins of the
    currently running host, publishing anything picked up
    during collection.

    Arguments:
        context (Context, optional): Context, defaults to
            creating a new context
        plugins (list, optional): Plug-ins to include,
            defaults to results of discover()
        targets (list, optional): Targets to include for publish session.

    Usage:
        >> context = plugin.Context()
        >> publish(context)     # Pass..
        >> context = publish()  # ..or receive a new

    """

    # Must check against None, as objects be emptys
    context = api.Context() if context is None else context
    plugins = api.discover() if plugins is None else plugins

    # Register targets
    for target in targets:
        api.register_target(target)

    # Do not consider inactive plug-ins
    plugins = list(p for p in plugins if p.active)
    collectors = list(p for p in plugins if lib.inrange(
        number=p.order,
        base=api.CollectorOrder)
    )

    # First pass, collection
    for Plugin, instance in logic.Iterator(collectors, context):
        plugin.process(Plugin, context, instance)

    # Exclude collectors from further processing
    plugins = list(p for p in plugins if p not in collectors)

    # Exclude plug-ins that do not have at
    # least one compatible instance.
    for Plugin in list(plugins):
        if Plugin.__instanceEnabled__:
            if not logic.instances_by_plugin(context, Plugin):
                plugins.remove(Plugin)

    # Mutable state, used in Iterator
    state = {
        "nextOrder": None,
        "ordersWithError": set()
    }

    # Second pass, the remainder
    for Plugin, instance in logic.Iterator(plugins, context, state):
        try:
            result = plugin.process(Plugin, context, instance)

        except StopIteration:  # End of items
            raise

        except:  # This is unexpected, most likely a bug
            log.error("An exception occurred.\n")
            raise

        else:
            # Make note of the order at which the
            # potential error error occured.
            if result["error"]:
                state["ordersWithError"].add(Plugin.order)

        if isinstance(result, Exception):
            log.error("An unexpected error happened: %s" % result)
            break

        error = result["error"]
        if error is not None:
            print(error)

    api.emit("published", context=context)

    # Deregister targets
    for target in targets:
        api.deregister_target(target)

    return context


def collect(context=None, plugins=None, targets=[]):
    """Convenience function for collection-only

     _________    . . . . .  .   . . . . . .   . . . . . . .
    |         |   .          .   .         .   .           .
    | Collect |-->. Validate .-->. Extract .-->. Integrate .
    |_________|   . . . . .  .   . . . . . .   . . . . . . .

    """

    context = _convenience(api.CollectorOrder, context, plugins, targets)
    api.emit("collected", context=context)
    return context


def validate(context=None, plugins=None, targets=[]):
    """Convenience function for validation-only

    . . . . . .    __________    . . . . . .   . . . . . . .
    .         .   |          |   .         .   .           .
    . Collect .-->| Validate |-->. Extract .-->. Integrate .
    . . . . . .   |__________|   . . . . . .   . . . . . . .

    """

    context = _convenience(api.ValidatorOrder, context, plugins, targets)
    api.emit("validated", context=context)
    return context


def extract(context=None, plugins=None, targets=[]):
    """Convenience function for extraction-only

    . . . . . .   . . . . .  .    _________    . . . . . . .
    .         .   .          .   |         |   .           .
    . Collect .-->. Validate .-->| Extract |-->. Integrate .
    . . . . . .   . . . . .  .   |_________|   . . . . . . .

    """

    context = _convenience(api.ExtractorOrder, context, plugins, targets)
    api.emit("extracted", context=context)
    return context


def integrate(context=None, plugins=None, targets=[]):
    """Convenience function for integration-only

    . . . . . .   . . . . .  .   . . . . . .    ___________
    .         .   .          .   .         .   |           |
    . Collect .-->. Validate .-->. Extract .-->| Integrate |
    . . . . . .   . . . . .  .   . . . . . .   |___________|

    """

    context = _convenience(api.IntegratorOrder, context, plugins, targets)
    api.emit("integrated", context=context)
    return context


def _convenience(order, context=None, plugins=None, targets=[]):
    plugins = list(
        p for p in (api.discover() if plugins is None else plugins)
        if lib.inrange(p.order, order)
    )

    return publish(context, plugins, targets)


# Backwards compatibility
select = collect
conform = integrate
run = publish  # Alias


def publish_all(context=None, plugins=None):
    warnings.warn("pyblish.util.publish_all has been "
                  "deprecated; use publish()")
    return publish(context, plugins)


def validate_all(context=None, plugins=None):
    warnings.warn("pyblish.util.validate_all has been "
                  "deprecated; use collect() followed by validate()")
    context = collect(context, plugins)
    return validate(context, plugins)



import pyblish.api
import pyblish.logic
import pyblish.plugin

from nose.tools import (
    assert_equals,
    with_setup
)

from . import lib


@with_setup(lib.setup_empty, lib.teardown)
def test_simple_discover():
    """Simple plug-ins works well with discover()"""

    count = {"#": 0}

    class SimplePlugin(pyblish.api.Plugin):
        def process(self, context):
            self.log.info("Processing context..")
            self.log.info("Done!")
            count["#"] += 1

    class SimplePlugin2(pyblish.api.Validator):
        def process(self, context):
            self.log.info("Processing context..")
            self.log.info("Done!")
            count["#"] += 1

    pyblish.api.register_plugin(SimplePlugin)
    pyblish.api.register_plugin(SimplePlugin2)

    assert_equals(
        list(p.id for p in pyblish.api.discover()),
        list(p.id for p in [SimplePlugin, SimplePlugin2])
    )

    pyblish.util.publish()

    assert_equals(count["#"], 2)


def test_simple_manual():
    """Simple plug-ins work well"""

    count = {"#": 0}

    class SimplePlugin(pyblish.api.Plugin):
        def process(self):
            self.log.info("Processing..")
            self.log.info("Done!")
            count["#"] += 1

    pyblish.util.publish(plugins=[SimplePlugin])

    assert_equals(count["#"], 1)


def test_simple_instance():
    """Simple plug-ins process instances as usual

    But considering they don't have an order, we will have to
    manually enforce an ordering if we are to expect
    them to run one after the other.

    """

    count = {"#": 0}

    class SimpleSelector(pyblish.api.Plugin):
        """Runs once"""
        order = 0

        def process(self, context):
            instance = context.create_instance(name="A")
            instance.set_data("family", "familyA")

            instance = context.create_instance(name="B")
            instance.set_data("family", "familyB")

            count["#"] += 1

    class SimpleValidator(pyblish.api.Plugin):
        """Runs twice"""
        order = 1

        def process(self, instance):
            count["#"] += 10

    class SimpleValidatorForB(pyblish.api.Plugin):
        """Runs once, for familyB"""
        families = ["familyB"]
        order = 2

        def process(self, instance):
            count["#"] += 100

    pyblish.util.publish(plugins=[SimpleSelector,
                                  SimpleValidator,
                                  SimpleValidatorForB])

    assert_equals(count["#"], 121)


def test_simple_order():
    """Simple plug-ins defaults to running *before* SVEC"""

    order = list()

    class SimplePlugin(pyblish.api.Plugin):
        def process(self):
            order.append(1)

    class SelectSomething1234(pyblish.api.Selector):
        def process(self):
            order.append(2)

    class ValidateSomething1234(pyblish.api.Validator):
        def process(self):
            order.append(3)

    class ExtractSomething1234(pyblish.api.Extractor):
        def process(self):
            order.append(4)

    for plugin in (ExtractSomething1234,
                   ValidateSomething1234,
                   SimplePlugin,
                   SelectSomething1234):
        pyblish.api.register_plugin(plugin)

    plugins = pyblish.api.discover()
    context = pyblish.api.Context()
    for result in pyblish.logic.process(
            func=pyblish.plugin.process,
            plugins=plugins,
            context=context):
        print(result)

    assert_equals(order, [1, 2, 3, 4])




import pyblish.api
import pyblish.lib


class CollectCurrentDate(pyblish.api.ContextPlugin):
    """Inject the current time into the Context"""

    order = pyblish.api.CollectorOrder
    label = "Current date"

    def process(self, context):
        context.data['date'] = pyblish.lib.time()



import os
import shutil
import tempfile

import pyblish.api
import pyblish.plugin
from nose.tools import (
    with_setup,
    assert_true,
)

from pyblish.vendor import six

from .. import lib


def test_plugins_from_module():
    """Getting plug-ins from a module works well"""
    import types

    module = types.ModuleType("myplugin")
    code = """
import pyblish.api

class MyPlugin(pyblish.api.Plugin):
    def process(self, context):
        pass

class NotSubclassed(object):
    def process(self, context):
        pass

def not_a_plugin():
    pass


class InvalidPlugin(pyblish.api.Plugin):
    families = False


class NotCompatible(pyblish.api.Plugin):
    hosts = ["not_compatible"]


class BadRequires(pyblish.api.Plugin):
    requires = None


class BadHosts(pyblish.api.Plugin):
    hosts = None


class BadFamilies(pyblish.api.Plugin):
    families = None


class BadHosts2(pyblish.api.Plugin):
    hosts = [None]


class BadFamilies2(pyblish.api.Plugin):
    families = [None]


"""

    six.exec_(code, module.__dict__)

    plugins = pyblish.plugin.plugins_from_module(module)

    assert [p.__name__ for p in plugins] == ["MyPlugin"], plugins


@with_setup(lib.setup_empty, lib.teardown)
def test_discover_globals():
    """Modules imported in a plug-in are preserved in it's methods"""

    import types

    module = types.ModuleType("myplugin")
    code = """
import pyblish.api
import threading

local_variable_is_present = 5


class MyPlugin(pyblish.api.Plugin):
    def module_is_present(self):
        return True if threading else False

    def local_variable_is_present(self):
        return True if local_variable_is_present else False

    def process(self, context):
        return True if context else False

"""

    six.exec_(code, module.__dict__)
    MyPlugin = pyblish.plugin.plugins_from_module(module)[0]
    assert MyPlugin.__name__ == "MyPlugin"

    assert_true(MyPlugin().process(True))
    assert_true(MyPlugin().module_is_present())
    assert_true(MyPlugin().local_variable_is_present())

    try:
        tempdir = tempfile.mkdtemp()
        tempplugin = os.path.join(tempdir, "my_plugin.py")
        with open(tempplugin, "w") as f:
            f.write(code)

        pyblish.api.register_plugin_path(tempdir)
        plugins = pyblish.api.discover()

    finally:
        shutil.rmtree(tempdir)

    assert len(plugins) == 1

    MyPlugin = plugins[0]

    assert_true(MyPlugin().process(True))
    assert_true(MyPlugin().module_is_present())
    assert_true(MyPlugin().local_variable_is_present())


@with_setup(lib.setup_empty, lib.teardown)
def test_multi_families():
    """Instances with multiple families works well"""

    count = {"#": 0}

    class CollectInstance(pyblish.api.Collector):
        def process(self, context):
            instance = context.create_instance("MyInstance")
            instance.data["families"] = ["geometry", "human"]

    class ValidateHumans(pyblish.api.Validator):
        families = ["human"]

        def process(self, instance):
            assert "human" in instance.data["families"]
            count["#"] += 10

    class ValidateGeometry(pyblish.api.Validator):
        families = ["geometry"]

        def process(self, instance):
            assert "geometry" in instance.data["families"]
            count["#"] += 100

    for plugin in (CollectInstance, ValidateHumans, ValidateGeometry):
        pyblish.api.register_plugin(plugin)

    pyblish.util.publish()

    assert count["#"] == 110, count["#"]



import os
import imp

from setuptools import setup, find_packages

with open("README.txt") as f:
    readme = f.read()


version_file = os.path.abspath("pyblish/version.py")
version_mod = imp.load_source("version", version_file)
version = version_mod.version


classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.6",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.1",
    "Programming Language :: Python :: 3.2",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Utilities"
]


setup(
    name="pyblish-base",
    version=version,
    description="Plug-in driven automation framework for content",
    long_description=readme,
    author="Abstract Factory and Contributors",
    author_email="marcus@abstractfactory.io",
    url="https://github.com/pyblish/pyblish",
    license="LGPL",
    packages=find_packages(),
    zip_safe=False,
    classifiers=classifiers,
    package_data={
        "pyblish": ["plugins/*.py",
                    "*.yaml",
                    "icons/*.svg"],
    },
    entry_points={
        "console_scripts": ["pyblish = pyblish.cli:main"]
    },
)
